local rand = require 'random'
module(..., package.seeall)

local messages = {
	"Spinning CPU cycles ...",
	"Releasing watchdog timer ...",
	"Cleaning dirty bits ...",
	"Sharpening instruction pointer ...",
	"Cashing registers ...",
	"Smoothing clock edge ...",
	"Contracting interrupt handlers ...",
}

function table.shuffle(tbl)
	for i = 2,#tbl do
		local k = rand.int(1,i)
		tbl[k], tbl[i] = tbl[i], tbl[k]
	end
end

local function make_texture(width,height)
	local id = love.image.newImageData(width,height)
	local function r() return math.max(0,math.min(255, rand.gauss(150))) end
	id:mapPixel(function() return r(),r(),r(),r() end)
	return id
end

local function interpolate(x,y,l,p)
	local p = p or 1
	local l = ((1 + math.cos(l * math.pi)) / 2)^p
	return l * x + (1-l) * y
end

local function value_noise(width, height, persistence, omin, omax)
	local noise_tex = make_texture(width,height)
	img = love.image.newImageData(width,height)
	for px = 0,width-1 do
		for py = 0,height-1 do
			local val = 0
			if (2*px/width-1)^2 + (2*py/height-1)^2 <= 1 then
				for i = omin,omax do
					local x,y = 2^i * px/width, 2^i * py/height
					local int_x,int_y = math.floor(x), math.floor(y)

					local value1 = noise_tex:getPixel(int_x     % width, int_y     % height)
					local value2 = noise_tex:getPixel((int_x+1) % width, int_y     % height)
					local value3 = noise_tex:getPixel(int_x     % width, (int_y+1) % height)
					local value4 = noise_tex:getPixel((int_x+1) % width, (int_y+1) % height)
					local inter = interpolate( interpolate(value1,value2, x%1), interpolate(value3,value4, x%1), y%1 )
					val = inter * persistence^i + val
				end
			end
			img:setPixel(px,py,math.min(val,255),0,0,0)
		end
	end
	return img
end

local function hsv(h,s,v)
	local c = v * s
	local H = h / 60
	local x = c * (1 - math.abs(H % 2 - 1))
	local m = v - c

	if     H >= 6 then return c+m, c+m, c+m
	elseif H >= 5 then return c+m, m,   x+m
	elseif H >= 4 then return x+m, m,   c+m
	elseif H >= 3 then return m,   x+m, c+m
	elseif H >= 2 then return m,   c+m, x+m
	elseif H >= 1 then return x+m, c+m, m
	end
	return c+m, x+m, m
end

local function post_process(id)
	local width,height = id:getWidth(), id:getHeight()
	id:mapPixel( function(x,y,a)
		local x,y = 2*x/width - 1, 2*y/height - 1
		local d = math.cos(math.max(0, 1 - (x*x+y*y)) * math.pi)
		local r,g,b,a = 0,0,0,a/255 * (.5 - d/2)^2
		if a > 0 then -- only colorize if pixel visible
			local h = interpolate(0,  50, a, 3)
			local s = interpolate(.88,.3, a, 4)
			local v = interpolate(.5, 1,  a, 2)
			r,g,b = hsv(h,s,v)
		end
		return r*255,g*255,b*255,a*255
	end)
	return id
end

sprites = { fire={} }
local fonts = {}
local generator, on_finish
local progress,message = 0, ""
function init(num_sprites, on_finish_)
	on_finish = on_finish_
	table.shuffle(messages)
	generator = coroutine.create(function()
		progress, message = 0, messages[1]
		for i = 1,num_sprites do
			coroutine.yield()
			sprites.fire[i] = love.graphics.newImage(post_process(
				value_noise(sprite_size, sprite_size, rand.uniform(.65,.8), 2, 8)))
			progress, message = i/num_sprites, messages[i%#messages+1]
		end
	end)
	fonts[20] = love.graphics.newFont(20)
	fonts[50] = love.graphics.newFont(50)
end

function update()
	if not coroutine.resume(generator) then
		on_finish()
	end
end

function draw()
	local sw,sh = love.graphics.getWidth(), love.graphics.getHeight()
	love.graphics.setFont(fonts[50])
	love.graphics.setColor(255,255,255)
	love.graphics.printf("vrld//loader", 100, (sh-230)/2, sw-200, "center")

	-- progress bar
	love.graphics.rectangle('fill', 100-6, (sh-20)/2-6, sw-200+12, 30+12)
	love.graphics.setColor(0,0,0)
	love.graphics.rectangle('fill', 100-4, (sh-20)/2-4, sw-200+8, 30+8)
	love.graphics.setColor(255,255,255)
	love.graphics.rectangle('fill', 100, (sh-20)/2, progress * (sw - 200), 30)

	-- messages
	love.graphics.setFont(fonts[20])
	love.graphics.printf(message, 100, (sh+100)/2, sw-200, "center")
end

function asplode() print("BOOM") end
